import time

from .crypto import PrivateKey, PublicKey
from .pb.CasperMessage_pb2 import DeployDataProto


def _gen_deploy_sig_content(data: DeployDataProto) -> bytes:
    signed_data = DeployDataProto()
    signed_data.term = data.term
    signed_data.timestamp = data.timestamp
    signed_data.phloLimit = data.phloLimit
    signed_data.phloPrice = data.phloPrice
    signed_data.validAfterBlockNumber = data.validAfterBlockNumber
    return signed_data.SerializeToString()


def sign_deploy_data(key: PrivateKey, data: DeployDataProto) -> bytes:
    return key.sign(_gen_deploy_sig_content(data))


def verify_deploy_data(key: PublicKey, sig: bytes, data: DeployDataProto) -> bool:
    return key.verify(sig, _gen_deploy_sig_content(data))


def create_deploy_data(
        key: PrivateKey,
        term: str,
        phlo_price: int,
        phlo_limit: int,
        valid_after_block_no: int = -1,
        timestamp_millis: int = -1,
) -> DeployDataProto:
    if timestamp_millis < 0:
        timestamp_millis = int(time.time() * 1000)
    data = DeployDataProto(
        deployer=key.get_public_key().to_bytes(),
        term=term,
        phloPrice=phlo_price,
        phloLimit=phlo_limit,
        validAfterBlockNumber=valid_after_block_no,
        timestamp=timestamp_millis,
        sigAlgorithm='secp256k1',
    )
    data.sig = sign_deploy_data(key, data)
    return data



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    Descriptor as google___protobuf___descriptor___Descriptor,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Optional as typing___Optional,
    Text as typing___Text,
    Union as typing___Union,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


builtin___bool = bool
builtin___bytes = bytes
builtin___float = float
builtin___int = int
if sys.version_info < (3,):
    builtin___buffer = buffer
    builtin___unicode = unicode


class Node(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    id = ... # type: builtin___bytes
    host = ... # type: builtin___bytes
    tcp_port = ... # type: builtin___int
    udp_port = ... # type: builtin___int

    def __init__(self,
        *,
        id : typing___Optional[builtin___bytes] = None,
        host : typing___Optional[builtin___bytes] = None,
        tcp_port : typing___Optional[builtin___int] = None,
        udp_port : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Node: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Node: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"host",b"host",u"id",b"id",u"tcp_port",b"tcp_port",u"udp_port",b"udp_port"]) -> None: ...

class Header(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    networkId = ... # type: typing___Text

    @property
    def sender(self) -> Node: ...

    def __init__(self,
        *,
        sender : typing___Optional[Node] = None,
        networkId : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Header: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Header: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"sender",b"sender"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"networkId",b"networkId",u"sender",b"sender"]) -> None: ...

class Heartbeat(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    def __init__(self,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Heartbeat: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Heartbeat: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class HeartbeatResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    def __init__(self,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> HeartbeatResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> HeartbeatResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class ProtocolHandshake(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    nonce = ... # type: builtin___bytes

    def __init__(self,
        *,
        nonce : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ProtocolHandshake: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ProtocolHandshake: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"nonce",b"nonce"]) -> None: ...

class ProtocolHandshakeResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    nonce = ... # type: builtin___bytes

    def __init__(self,
        *,
        nonce : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ProtocolHandshakeResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ProtocolHandshakeResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"nonce",b"nonce"]) -> None: ...

class Packet(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    typeId = ... # type: typing___Text
    content = ... # type: builtin___bytes

    def __init__(self,
        *,
        typeId : typing___Optional[typing___Text] = None,
        content : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Packet: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Packet: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"content",b"content",u"typeId",b"typeId"]) -> None: ...

class Disconnect(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    def __init__(self,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Disconnect: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Disconnect: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class Protocol(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def header(self) -> Header: ...

    @property
    def heartbeat(self) -> Heartbeat: ...

    @property
    def protocol_handshake(self) -> ProtocolHandshake: ...

    @property
    def protocol_handshake_response(self) -> ProtocolHandshakeResponse: ...

    @property
    def packet(self) -> Packet: ...

    @property
    def disconnect(self) -> Disconnect: ...

    def __init__(self,
        *,
        header : typing___Optional[Header] = None,
        heartbeat : typing___Optional[Heartbeat] = None,
        protocol_handshake : typing___Optional[ProtocolHandshake] = None,
        protocol_handshake_response : typing___Optional[ProtocolHandshakeResponse] = None,
        packet : typing___Optional[Packet] = None,
        disconnect : typing___Optional[Disconnect] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Protocol: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Protocol: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"disconnect",b"disconnect",u"header",b"header",u"heartbeat",b"heartbeat",u"message",b"message",u"packet",b"packet",u"protocol_handshake",b"protocol_handshake",u"protocol_handshake_response",b"protocol_handshake_response"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"disconnect",b"disconnect",u"header",b"header",u"heartbeat",b"heartbeat",u"message",b"message",u"packet",b"packet",u"protocol_handshake",b"protocol_handshake",u"protocol_handshake_response",b"protocol_handshake_response"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["heartbeat","protocol_handshake","protocol_handshake_response","packet","disconnect"]: ...

class TLRequest(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def protocol(self) -> Protocol: ...

    def __init__(self,
        *,
        protocol : typing___Optional[Protocol] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> TLRequest: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> TLRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"protocol",b"protocol"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"protocol",b"protocol"]) -> None: ...

class InternalServerError(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    error = ... # type: builtin___bytes

    def __init__(self,
        *,
        error : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> InternalServerError: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> InternalServerError: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"error",b"error"]) -> None: ...

class Ack(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def header(self) -> Header: ...

    def __init__(self,
        *,
        header : typing___Optional[Header] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Ack: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Ack: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"header",b"header"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"header",b"header"]) -> None: ...

class TLResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def ack(self) -> Ack: ...

    @property
    def internalServerError(self) -> InternalServerError: ...

    def __init__(self,
        *,
        ack : typing___Optional[Ack] = None,
        internalServerError : typing___Optional[InternalServerError] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> TLResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> TLResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"ack",b"ack",u"internalServerError",b"internalServerError",u"payload",b"payload"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"ack",b"ack",u"internalServerError",b"internalServerError",u"payload",b"payload"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"payload",b"payload"]) -> typing_extensions___Literal["ack","internalServerError"]: ...

class ChunkHeader(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    typeId = ... # type: typing___Text
    compressed = ... # type: builtin___bool
    contentLength = ... # type: builtin___int
    networkId = ... # type: typing___Text

    @property
    def sender(self) -> Node: ...

    def __init__(self,
        *,
        sender : typing___Optional[Node] = None,
        typeId : typing___Optional[typing___Text] = None,
        compressed : typing___Optional[builtin___bool] = None,
        contentLength : typing___Optional[builtin___int] = None,
        networkId : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ChunkHeader: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ChunkHeader: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"sender",b"sender"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"compressed",b"compressed",u"contentLength",b"contentLength",u"networkId",b"networkId",u"sender",b"sender",u"typeId",b"typeId"]) -> None: ...

class ChunkData(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    contentData = ... # type: builtin___bytes

    def __init__(self,
        *,
        contentData : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ChunkData: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ChunkData: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"contentData",b"contentData"]) -> None: ...

class Chunk(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def header(self) -> ChunkHeader: ...

    @property
    def data(self) -> ChunkData: ...

    def __init__(self,
        *,
        header : typing___Optional[ChunkHeader] = None,
        data : typing___Optional[ChunkData] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Chunk: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Chunk: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"content",b"content",u"data",b"data",u"header",b"header"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"content",b"content",u"data",b"data",u"header",b"header"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"content",b"content"]) -> typing_extensions___Literal["header","data"]: ...



from dataclasses import (
    _FIELD, _FIELD_INITVAR, _FIELDS, _HAS_DEFAULT_FACTORY, MISSING, Field,
    _create_fn, _set_new_attribute,
)
from typing import List

_PB_PARAM = "PB"
_FROM_PB = "__from_pb"


def _make_from_pb_fn(fields: List[Field]):
    globals = {'MISSING': MISSING,
               '_HAS_DEFAULT_FACTORY': _HAS_DEFAULT_FACTORY}
    locals = {f'_type_{f.name}': f.type for f in fields}
    globals.update({f'_pb_type_{f.name}': f.default_factory for f in fields if f.default_factory != MISSING})
    body_lines = []
    for field in fields:
        field_type_name = getattr(field.type, "_name", None)
        if field_type_name == "List":
            if field.default_factory != MISSING:
                body_lines.append(
                    "{}=[_pb_type_{}.from_pb(_pb) for _pb in {}.{}],".format(field.name, field.name, _PB_PARAM,
                                                                             field.name))
            else:
                body_lines.append("{}=[i for i in {}.{}],".format(field.name, _PB_PARAM, field.name))
        elif getattr(field.type, _FROM_PB, None):
            if field.default_factory == MISSING:
                raise ValueError("from pb class must provide a default factory")
            else:
                body_lines.append(
                    "{}=_pb_type_{}.from_pb({}.{}),".format(field.name, field.name, _PB_PARAM, field.name))
        else:
            body_lines.append("{}={}.{},".format(field.name, _PB_PARAM, field.name))
    body_lines.insert(0, "return cls(")
    body_lines.append(")")
    return _create_fn("from_pb", ["cls", _PB_PARAM], body_lines, globals=globals,
                      locals=locals)


def _process_cls(cls):
    fields = getattr(cls, _FIELDS, {})
    flds = [f for f in fields.values()
            if f._field_type in (_FIELD, _FIELD_INITVAR)]
    fn = _make_from_pb_fn(flds)
    _set_new_attribute(cls, "from_pb", classmethod(fn))
    _set_new_attribute(cls, _FROM_PB, True)
    return cls


def from_pb(_cls):
    """
    Make sure _cls is a wrap by dataclass.
    Generate a from_pb class method for the dataclass based
    on the dataclass definition.
    """

    def wrap(cls):
        return _process_cls(cls)

    if _cls is None:
        return wrap

    return wrap(_cls)



from dataclasses import dataclass
from typing import List

from .meta import from_pb


@from_pb
@dataclass
class DeployInfo:
    deployer: str
    term: str
    timestamp: int
    sig: str
    sigAlgorithm: str
    phloPrice: int
    phloLimit: int
    validAfterBlockNumber: int
    cost: int
    errored: bool
    systemDeployError: str


@from_pb
@dataclass
class BondInfo:
    validator: str
    stake: int


@from_pb
@dataclass
class LightBlockInfo:
    # BlockMessageProto
    blockHash: str
    sender: str
    seqNum: int
    sig: str
    sigAlgorithm: str
    shardId: str
    extraBytes: bytes

    # HeaderProto
    version: int
    timestamp: int
    headerExtraBytes: bytes
    parentsHashList: List[str]

    # BodyProto
    blockNumber: int
    preStateHash: str
    postStateHash: str
    bodyExtraBytes: bytes
    BondInfo: List[BondInfo]

    # extra
    blockSize: str
    deployCount: int
    faultTolerance: float


@from_pb
@dataclass
class BlockInfo:
    blockInfo: LightBlockInfo
    deploys: List[DeployInfo]



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    Descriptor as google___protobuf___descriptor___Descriptor,
    EnumDescriptor as google___protobuf___descriptor___EnumDescriptor,
    FieldDescriptor as google___protobuf___descriptor___FieldDescriptor,
)

from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer as google___protobuf___internal___containers___RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer as google___protobuf___internal___containers___RepeatedScalarFieldContainer,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Iterable as typing___Iterable,
    List as typing___List,
    Optional as typing___Optional,
    Text as typing___Text,
    Tuple as typing___Tuple,
    Union as typing___Union,
    cast as typing___cast,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


builtin___bool = bool
builtin___bytes = bytes
builtin___float = float
builtin___int = int
builtin___str = str
if sys.version_info < (3,):
    builtin___buffer = buffer
    builtin___unicode = unicode


class ScalaPbOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    class OptionsScope(builtin___int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: builtin___int) -> builtin___str: ...
        @classmethod
        def Value(cls, name: builtin___str) -> 'ScalaPbOptions.OptionsScope': ...
        @classmethod
        def keys(cls) -> typing___List[builtin___str]: ...
        @classmethod
        def values(cls) -> typing___List['ScalaPbOptions.OptionsScope']: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[builtin___str, 'ScalaPbOptions.OptionsScope']]: ...
        FILE = typing___cast('ScalaPbOptions.OptionsScope', 0)
        PACKAGE = typing___cast('ScalaPbOptions.OptionsScope', 1)
    FILE = typing___cast('ScalaPbOptions.OptionsScope', 0)
    PACKAGE = typing___cast('ScalaPbOptions.OptionsScope', 1)

    class EnumValueNaming(builtin___int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: builtin___int) -> builtin___str: ...
        @classmethod
        def Value(cls, name: builtin___str) -> 'ScalaPbOptions.EnumValueNaming': ...
        @classmethod
        def keys(cls) -> typing___List[builtin___str]: ...
        @classmethod
        def values(cls) -> typing___List['ScalaPbOptions.EnumValueNaming']: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[builtin___str, 'ScalaPbOptions.EnumValueNaming']]: ...
        AS_IN_PROTO = typing___cast('ScalaPbOptions.EnumValueNaming', 0)
        CAMEL_CASE = typing___cast('ScalaPbOptions.EnumValueNaming', 1)
    AS_IN_PROTO = typing___cast('ScalaPbOptions.EnumValueNaming', 0)
    CAMEL_CASE = typing___cast('ScalaPbOptions.EnumValueNaming', 1)

    class AuxMessageOptions(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        target = ... # type: typing___Text

        @property
        def options(self) -> MessageOptions: ...

        def __init__(self,
            *,
            target : typing___Optional[typing___Text] = None,
            options : typing___Optional[MessageOptions] = None,
            ) -> None: ...
        if sys.version_info >= (3,):
            @classmethod
            def FromString(cls, s: builtin___bytes) -> ScalaPbOptions.AuxMessageOptions: ...
        else:
            @classmethod
            def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ScalaPbOptions.AuxMessageOptions: ...
        def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def HasField(self, field_name: typing_extensions___Literal[u"options",b"options",u"target",b"target"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"options",b"options",u"target",b"target"]) -> None: ...

    class AuxFieldOptions(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        target = ... # type: typing___Text

        @property
        def options(self) -> FieldOptions: ...

        def __init__(self,
            *,
            target : typing___Optional[typing___Text] = None,
            options : typing___Optional[FieldOptions] = None,
            ) -> None: ...
        if sys.version_info >= (3,):
            @classmethod
            def FromString(cls, s: builtin___bytes) -> ScalaPbOptions.AuxFieldOptions: ...
        else:
            @classmethod
            def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ScalaPbOptions.AuxFieldOptions: ...
        def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def HasField(self, field_name: typing_extensions___Literal[u"options",b"options",u"target",b"target"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"options",b"options",u"target",b"target"]) -> None: ...

    class AuxEnumOptions(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        target = ... # type: typing___Text

        @property
        def options(self) -> EnumOptions: ...

        def __init__(self,
            *,
            target : typing___Optional[typing___Text] = None,
            options : typing___Optional[EnumOptions] = None,
            ) -> None: ...
        if sys.version_info >= (3,):
            @classmethod
            def FromString(cls, s: builtin___bytes) -> ScalaPbOptions.AuxEnumOptions: ...
        else:
            @classmethod
            def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ScalaPbOptions.AuxEnumOptions: ...
        def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def HasField(self, field_name: typing_extensions___Literal[u"options",b"options",u"target",b"target"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"options",b"options",u"target",b"target"]) -> None: ...

    package_name = ... # type: typing___Text
    flat_package = ... # type: builtin___bool
    preamble = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]
    single_file = ... # type: builtin___bool
    no_primitive_wrappers = ... # type: builtin___bool
    primitive_wrappers = ... # type: builtin___bool
    collection_type = ... # type: typing___Text
    preserve_unknown_fields = ... # type: builtin___bool
    object_name = ... # type: typing___Text
    scope = ... # type: ScalaPbOptions.OptionsScope
    lenses = ... # type: builtin___bool
    retain_source_code_info = ... # type: builtin___bool
    map_type = ... # type: typing___Text
    no_default_values_in_constructor = ... # type: builtin___bool
    enum_value_naming = ... # type: ScalaPbOptions.EnumValueNaming
    enum_strip_prefix = ... # type: builtin___bool
    bytes_type = ... # type: typing___Text
    test_only_no_java_conversions = ... # type: builtin___bool

    @property
    def aux_message_options(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[ScalaPbOptions.AuxMessageOptions]: ...

    @property
    def aux_field_options(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[ScalaPbOptions.AuxFieldOptions]: ...

    @property
    def aux_enum_options(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[ScalaPbOptions.AuxEnumOptions]: ...

    def __init__(self,
        *,
        package_name : typing___Optional[typing___Text] = None,
        flat_package : typing___Optional[builtin___bool] = None,
        preamble : typing___Optional[typing___Iterable[typing___Text]] = None,
        single_file : typing___Optional[builtin___bool] = None,
        no_primitive_wrappers : typing___Optional[builtin___bool] = None,
        primitive_wrappers : typing___Optional[builtin___bool] = None,
        collection_type : typing___Optional[typing___Text] = None,
        preserve_unknown_fields : typing___Optional[builtin___bool] = None,
        object_name : typing___Optional[typing___Text] = None,
        scope : typing___Optional[ScalaPbOptions.OptionsScope] = None,
        lenses : typing___Optional[builtin___bool] = None,
        retain_source_code_info : typing___Optional[builtin___bool] = None,
        map_type : typing___Optional[typing___Text] = None,
        no_default_values_in_constructor : typing___Optional[builtin___bool] = None,
        enum_value_naming : typing___Optional[ScalaPbOptions.EnumValueNaming] = None,
        enum_strip_prefix : typing___Optional[builtin___bool] = None,
        aux_message_options : typing___Optional[typing___Iterable[ScalaPbOptions.AuxMessageOptions]] = None,
        aux_field_options : typing___Optional[typing___Iterable[ScalaPbOptions.AuxFieldOptions]] = None,
        aux_enum_options : typing___Optional[typing___Iterable[ScalaPbOptions.AuxEnumOptions]] = None,
        bytes_type : typing___Optional[typing___Text] = None,
        test_only_no_java_conversions : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ScalaPbOptions: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ScalaPbOptions: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"bytes_type",b"bytes_type",u"collection_type",b"collection_type",u"enum_strip_prefix",b"enum_strip_prefix",u"enum_value_naming",b"enum_value_naming",u"flat_package",b"flat_package",u"lenses",b"lenses",u"map_type",b"map_type",u"no_default_values_in_constructor",b"no_default_values_in_constructor",u"no_primitive_wrappers",b"no_primitive_wrappers",u"object_name",b"object_name",u"package_name",b"package_name",u"preserve_unknown_fields",b"preserve_unknown_fields",u"primitive_wrappers",b"primitive_wrappers",u"retain_source_code_info",b"retain_source_code_info",u"scope",b"scope",u"single_file",b"single_file",u"test_only_no_java_conversions",b"test_only_no_java_conversions"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"aux_enum_options",b"aux_enum_options",u"aux_field_options",b"aux_field_options",u"aux_message_options",b"aux_message_options",u"bytes_type",b"bytes_type",u"collection_type",b"collection_type",u"enum_strip_prefix",b"enum_strip_prefix",u"enum_value_naming",b"enum_value_naming",u"flat_package",b"flat_package",u"import",b"import",u"lenses",b"lenses",u"map_type",b"map_type",u"no_default_values_in_constructor",b"no_default_values_in_constructor",u"no_primitive_wrappers",b"no_primitive_wrappers",u"object_name",b"object_name",u"package_name",b"package_name",u"preamble",b"preamble",u"preserve_unknown_fields",b"preserve_unknown_fields",u"primitive_wrappers",b"primitive_wrappers",u"retain_source_code_info",b"retain_source_code_info",u"scope",b"scope",u"single_file",b"single_file",u"test_only_no_java_conversions",b"test_only_no_java_conversions"]) -> None: ...

class MessageOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    extends = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]
    companion_extends = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]
    annotations = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]
    type = ... # type: typing___Text
    companion_annotations = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]
    sealed_oneof_extends = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]
    no_box = ... # type: builtin___bool

    def __init__(self,
        *,
        extends : typing___Optional[typing___Iterable[typing___Text]] = None,
        companion_extends : typing___Optional[typing___Iterable[typing___Text]] = None,
        annotations : typing___Optional[typing___Iterable[typing___Text]] = None,
        type : typing___Optional[typing___Text] = None,
        companion_annotations : typing___Optional[typing___Iterable[typing___Text]] = None,
        sealed_oneof_extends : typing___Optional[typing___Iterable[typing___Text]] = None,
        no_box : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> MessageOptions: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> MessageOptions: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"no_box",b"no_box",u"type",b"type"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"annotations",b"annotations",u"companion_annotations",b"companion_annotations",u"companion_extends",b"companion_extends",u"extends",b"extends",u"no_box",b"no_box",u"sealed_oneof_extends",b"sealed_oneof_extends",u"type",b"type"]) -> None: ...

class FieldOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    type = ... # type: typing___Text
    scala_name = ... # type: typing___Text
    collection_type = ... # type: typing___Text
    key_type = ... # type: typing___Text
    value_type = ... # type: typing___Text
    annotations = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]
    map_type = ... # type: typing___Text
    no_box = ... # type: builtin___bool

    def __init__(self,
        *,
        type : typing___Optional[typing___Text] = None,
        scala_name : typing___Optional[typing___Text] = None,
        collection_type : typing___Optional[typing___Text] = None,
        key_type : typing___Optional[typing___Text] = None,
        value_type : typing___Optional[typing___Text] = None,
        annotations : typing___Optional[typing___Iterable[typing___Text]] = None,
        map_type : typing___Optional[typing___Text] = None,
        no_box : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> FieldOptions: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> FieldOptions: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"collection_type",b"collection_type",u"key_type",b"key_type",u"map_type",b"map_type",u"no_box",b"no_box",u"scala_name",b"scala_name",u"type",b"type",u"value_type",b"value_type"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"annotations",b"annotations",u"collection_type",b"collection_type",u"key_type",b"key_type",u"map_type",b"map_type",u"no_box",b"no_box",u"scala_name",b"scala_name",u"type",b"type",u"value_type",b"value_type"]) -> None: ...

class EnumOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    extends = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]
    companion_extends = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]
    type = ... # type: typing___Text

    def __init__(self,
        *,
        extends : typing___Optional[typing___Iterable[typing___Text]] = None,
        companion_extends : typing___Optional[typing___Iterable[typing___Text]] = None,
        type : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EnumOptions: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EnumOptions: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"type",b"type"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"companion_extends",b"companion_extends",u"extends",b"extends",u"type",b"type"]) -> None: ...

class EnumValueOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    extends = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]
    scala_name = ... # type: typing___Text

    def __init__(self,
        *,
        extends : typing___Optional[typing___Iterable[typing___Text]] = None,
        scala_name : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EnumValueOptions: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EnumValueOptions: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"scala_name",b"scala_name"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"extends",b"extends",u"scala_name",b"scala_name"]) -> None: ...

class OneofOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    extends = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]

    def __init__(self,
        *,
        extends : typing___Optional[typing___Iterable[typing___Text]] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> OneofOptions: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> OneofOptions: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"extends",b"extends"]) -> None: ...

options = ... # type: google___protobuf___descriptor___FieldDescriptor

message = ... # type: google___protobuf___descriptor___FieldDescriptor

field = ... # type: google___protobuf___descriptor___FieldDescriptor

enum_options = ... # type: google___protobuf___descriptor___FieldDescriptor

enum_value = ... # type: google___protobuf___descriptor___FieldDescriptor

oneof = ... # type: google___protobuf___descriptor___FieldDescriptor



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    Descriptor as google___protobuf___descriptor___Descriptor,
)

from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer as google___protobuf___internal___containers___RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer as google___protobuf___internal___containers___RepeatedScalarFieldContainer,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Iterable as typing___Iterable,
    Mapping as typing___Mapping,
    MutableMapping as typing___MutableMapping,
    Optional as typing___Optional,
    Text as typing___Text,
    Union as typing___Union,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


builtin___bool = bool
builtin___bytes = bytes
builtin___float = float
builtin___int = int
if sys.version_info < (3,):
    builtin___buffer = buffer
    builtin___unicode = unicode


class Par(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    locallyFree = ... # type: builtin___bytes
    connective_used = ... # type: builtin___bool

    @property
    def sends(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Send]: ...

    @property
    def receives(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Receive]: ...

    @property
    def news(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[New]: ...

    @property
    def exprs(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Expr]: ...

    @property
    def matches(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Match]: ...

    @property
    def unforgeables(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[GUnforgeable]: ...

    @property
    def bundles(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Bundle]: ...

    @property
    def connectives(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Connective]: ...

    def __init__(self,
        *,
        sends : typing___Optional[typing___Iterable[Send]] = None,
        receives : typing___Optional[typing___Iterable[Receive]] = None,
        news : typing___Optional[typing___Iterable[New]] = None,
        exprs : typing___Optional[typing___Iterable[Expr]] = None,
        matches : typing___Optional[typing___Iterable[Match]] = None,
        unforgeables : typing___Optional[typing___Iterable[GUnforgeable]] = None,
        bundles : typing___Optional[typing___Iterable[Bundle]] = None,
        connectives : typing___Optional[typing___Iterable[Connective]] = None,
        locallyFree : typing___Optional[builtin___bytes] = None,
        connective_used : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Par: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Par: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"bundles",b"bundles",u"connective_used",b"connective_used",u"connectives",b"connectives",u"exprs",b"exprs",u"locallyFree",b"locallyFree",u"matches",b"matches",u"news",b"news",u"receives",b"receives",u"sends",b"sends",u"unforgeables",b"unforgeables"]) -> None: ...

class TaggedContinuation(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    scala_body_ref = ... # type: builtin___int

    @property
    def par_body(self) -> ParWithRandom: ...

    def __init__(self,
        *,
        par_body : typing___Optional[ParWithRandom] = None,
        scala_body_ref : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> TaggedContinuation: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> TaggedContinuation: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"par_body",b"par_body",u"scala_body_ref",b"scala_body_ref",u"tagged_cont",b"tagged_cont"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"par_body",b"par_body",u"scala_body_ref",b"scala_body_ref",u"tagged_cont",b"tagged_cont"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"tagged_cont",b"tagged_cont"]) -> typing_extensions___Literal["par_body","scala_body_ref"]: ...

class ParWithRandom(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    randomState = ... # type: builtin___bytes

    @property
    def body(self) -> Par: ...

    def __init__(self,
        *,
        body : typing___Optional[Par] = None,
        randomState : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ParWithRandom: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ParWithRandom: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"body",b"body"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"body",b"body",u"randomState",b"randomState"]) -> None: ...

class PCost(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    cost = ... # type: builtin___int

    def __init__(self,
        *,
        cost : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> PCost: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> PCost: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"cost",b"cost"]) -> None: ...

class ListParWithRandom(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    randomState = ... # type: builtin___bytes

    @property
    def pars(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Par]: ...

    def __init__(self,
        *,
        pars : typing___Optional[typing___Iterable[Par]] = None,
        randomState : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ListParWithRandom: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ListParWithRandom: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"pars",b"pars",u"randomState",b"randomState"]) -> None: ...

class Var(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    class WildcardMsg(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

        def __init__(self,
            ) -> None: ...
        if sys.version_info >= (3,):
            @classmethod
            def FromString(cls, s: builtin___bytes) -> Var.WildcardMsg: ...
        else:
            @classmethod
            def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Var.WildcardMsg: ...
        def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

    bound_var = ... # type: builtin___int
    free_var = ... # type: builtin___int

    @property
    def wildcard(self) -> Var.WildcardMsg: ...

    def __init__(self,
        *,
        bound_var : typing___Optional[builtin___int] = None,
        free_var : typing___Optional[builtin___int] = None,
        wildcard : typing___Optional[Var.WildcardMsg] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Var: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Var: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"bound_var",b"bound_var",u"free_var",b"free_var",u"var_instance",b"var_instance",u"wildcard",b"wildcard"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"bound_var",b"bound_var",u"free_var",b"free_var",u"var_instance",b"var_instance",u"wildcard",b"wildcard"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"var_instance",b"var_instance"]) -> typing_extensions___Literal["bound_var","free_var","wildcard"]: ...

class Bundle(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    writeFlag = ... # type: builtin___bool
    readFlag = ... # type: builtin___bool

    @property
    def body(self) -> Par: ...

    def __init__(self,
        *,
        body : typing___Optional[Par] = None,
        writeFlag : typing___Optional[builtin___bool] = None,
        readFlag : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Bundle: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Bundle: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"body",b"body"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"body",b"body",u"readFlag",b"readFlag",u"writeFlag",b"writeFlag"]) -> None: ...

class Send(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    persistent = ... # type: builtin___bool
    locallyFree = ... # type: builtin___bytes
    connective_used = ... # type: builtin___bool

    @property
    def chan(self) -> Par: ...

    @property
    def data(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Par]: ...

    def __init__(self,
        *,
        chan : typing___Optional[Par] = None,
        data : typing___Optional[typing___Iterable[Par]] = None,
        persistent : typing___Optional[builtin___bool] = None,
        locallyFree : typing___Optional[builtin___bytes] = None,
        connective_used : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Send: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Send: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"chan",b"chan"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"chan",b"chan",u"connective_used",b"connective_used",u"data",b"data",u"locallyFree",b"locallyFree",u"persistent",b"persistent"]) -> None: ...

class ReceiveBind(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    freeCount = ... # type: builtin___int

    @property
    def patterns(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Par]: ...

    @property
    def source(self) -> Par: ...

    @property
    def remainder(self) -> Var: ...

    def __init__(self,
        *,
        patterns : typing___Optional[typing___Iterable[Par]] = None,
        source : typing___Optional[Par] = None,
        remainder : typing___Optional[Var] = None,
        freeCount : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ReceiveBind: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ReceiveBind: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"remainder",b"remainder",u"source",b"source"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"freeCount",b"freeCount",u"patterns",b"patterns",u"remainder",b"remainder",u"source",b"source"]) -> None: ...

class BindPattern(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    freeCount = ... # type: builtin___int

    @property
    def patterns(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Par]: ...

    @property
    def remainder(self) -> Var: ...

    def __init__(self,
        *,
        patterns : typing___Optional[typing___Iterable[Par]] = None,
        remainder : typing___Optional[Var] = None,
        freeCount : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BindPattern: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BindPattern: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"remainder",b"remainder"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"freeCount",b"freeCount",u"patterns",b"patterns",u"remainder",b"remainder"]) -> None: ...

class ListBindPatterns(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def patterns(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[BindPattern]: ...

    def __init__(self,
        *,
        patterns : typing___Optional[typing___Iterable[BindPattern]] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ListBindPatterns: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ListBindPatterns: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"patterns",b"patterns"]) -> None: ...

class Receive(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    persistent = ... # type: builtin___bool
    peek = ... # type: builtin___bool
    bindCount = ... # type: builtin___int
    locallyFree = ... # type: builtin___bytes
    connective_used = ... # type: builtin___bool

    @property
    def binds(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[ReceiveBind]: ...

    @property
    def body(self) -> Par: ...

    def __init__(self,
        *,
        binds : typing___Optional[typing___Iterable[ReceiveBind]] = None,
        body : typing___Optional[Par] = None,
        persistent : typing___Optional[builtin___bool] = None,
        peek : typing___Optional[builtin___bool] = None,
        bindCount : typing___Optional[builtin___int] = None,
        locallyFree : typing___Optional[builtin___bytes] = None,
        connective_used : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Receive: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Receive: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"body",b"body"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"bindCount",b"bindCount",u"binds",b"binds",u"body",b"body",u"connective_used",b"connective_used",u"locallyFree",b"locallyFree",u"peek",b"peek",u"persistent",b"persistent"]) -> None: ...

class New(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    class InjectionsEntry(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        key = ... # type: typing___Text

        @property
        def value(self) -> Par: ...

        def __init__(self,
            *,
            key : typing___Optional[typing___Text] = None,
            value : typing___Optional[Par] = None,
            ) -> None: ...
        if sys.version_info >= (3,):
            @classmethod
            def FromString(cls, s: builtin___bytes) -> New.InjectionsEntry: ...
        else:
            @classmethod
            def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> New.InjectionsEntry: ...
        def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"key",b"key",u"value",b"value"]) -> None: ...

    bindCount = ... # type: builtin___int
    uri = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]
    locallyFree = ... # type: builtin___bytes

    @property
    def p(self) -> Par: ...

    @property
    def injections(self) -> typing___MutableMapping[typing___Text, Par]: ...

    def __init__(self,
        *,
        bindCount : typing___Optional[builtin___int] = None,
        p : typing___Optional[Par] = None,
        uri : typing___Optional[typing___Iterable[typing___Text]] = None,
        injections : typing___Optional[typing___Mapping[typing___Text, Par]] = None,
        locallyFree : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> New: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> New: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p",b"p"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"bindCount",b"bindCount",u"injections",b"injections",u"locallyFree",b"locallyFree",u"p",b"p",u"uri",b"uri"]) -> None: ...

class MatchCase(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    freeCount = ... # type: builtin___int

    @property
    def pattern(self) -> Par: ...

    @property
    def source(self) -> Par: ...

    def __init__(self,
        *,
        pattern : typing___Optional[Par] = None,
        source : typing___Optional[Par] = None,
        freeCount : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> MatchCase: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> MatchCase: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"pattern",b"pattern",u"source",b"source"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"freeCount",b"freeCount",u"pattern",b"pattern",u"source",b"source"]) -> None: ...

class Match(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    locallyFree = ... # type: builtin___bytes
    connective_used = ... # type: builtin___bool

    @property
    def target(self) -> Par: ...

    @property
    def cases(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[MatchCase]: ...

    def __init__(self,
        *,
        target : typing___Optional[Par] = None,
        cases : typing___Optional[typing___Iterable[MatchCase]] = None,
        locallyFree : typing___Optional[builtin___bytes] = None,
        connective_used : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Match: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Match: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"target",b"target"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"cases",b"cases",u"connective_used",b"connective_used",u"locallyFree",b"locallyFree",u"target",b"target"]) -> None: ...

class Expr(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    g_bool = ... # type: builtin___bool
    g_int = ... # type: builtin___int
    g_string = ... # type: typing___Text
    g_uri = ... # type: typing___Text
    g_byte_array = ... # type: builtin___bytes

    @property
    def e_not_body(self) -> ENot: ...

    @property
    def e_neg_body(self) -> ENeg: ...

    @property
    def e_mult_body(self) -> EMult: ...

    @property
    def e_div_body(self) -> EDiv: ...

    @property
    def e_plus_body(self) -> EPlus: ...

    @property
    def e_minus_body(self) -> EMinus: ...

    @property
    def e_lt_body(self) -> ELt: ...

    @property
    def e_lte_body(self) -> ELte: ...

    @property
    def e_gt_body(self) -> EGt: ...

    @property
    def e_gte_body(self) -> EGte: ...

    @property
    def e_eq_body(self) -> EEq: ...

    @property
    def e_neq_body(self) -> ENeq: ...

    @property
    def e_and_body(self) -> EAnd: ...

    @property
    def e_or_body(self) -> EOr: ...

    @property
    def e_var_body(self) -> EVar: ...

    @property
    def e_list_body(self) -> EList: ...

    @property
    def e_tuple_body(self) -> ETuple: ...

    @property
    def e_set_body(self) -> ESet: ...

    @property
    def e_map_body(self) -> EMap: ...

    @property
    def e_method_body(self) -> EMethod: ...

    @property
    def e_matches_body(self) -> EMatches: ...

    @property
    def e_percent_percent_body(self) -> EPercentPercent: ...

    @property
    def e_plus_plus_body(self) -> EPlusPlus: ...

    @property
    def e_minus_minus_body(self) -> EMinusMinus: ...

    @property
    def e_mod_body(self) -> EMod: ...

    def __init__(self,
        *,
        g_bool : typing___Optional[builtin___bool] = None,
        g_int : typing___Optional[builtin___int] = None,
        g_string : typing___Optional[typing___Text] = None,
        g_uri : typing___Optional[typing___Text] = None,
        g_byte_array : typing___Optional[builtin___bytes] = None,
        e_not_body : typing___Optional[ENot] = None,
        e_neg_body : typing___Optional[ENeg] = None,
        e_mult_body : typing___Optional[EMult] = None,
        e_div_body : typing___Optional[EDiv] = None,
        e_plus_body : typing___Optional[EPlus] = None,
        e_minus_body : typing___Optional[EMinus] = None,
        e_lt_body : typing___Optional[ELt] = None,
        e_lte_body : typing___Optional[ELte] = None,
        e_gt_body : typing___Optional[EGt] = None,
        e_gte_body : typing___Optional[EGte] = None,
        e_eq_body : typing___Optional[EEq] = None,
        e_neq_body : typing___Optional[ENeq] = None,
        e_and_body : typing___Optional[EAnd] = None,
        e_or_body : typing___Optional[EOr] = None,
        e_var_body : typing___Optional[EVar] = None,
        e_list_body : typing___Optional[EList] = None,
        e_tuple_body : typing___Optional[ETuple] = None,
        e_set_body : typing___Optional[ESet] = None,
        e_map_body : typing___Optional[EMap] = None,
        e_method_body : typing___Optional[EMethod] = None,
        e_matches_body : typing___Optional[EMatches] = None,
        e_percent_percent_body : typing___Optional[EPercentPercent] = None,
        e_plus_plus_body : typing___Optional[EPlusPlus] = None,
        e_minus_minus_body : typing___Optional[EMinusMinus] = None,
        e_mod_body : typing___Optional[EMod] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Expr: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Expr: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"e_and_body",b"e_and_body",u"e_div_body",b"e_div_body",u"e_eq_body",b"e_eq_body",u"e_gt_body",b"e_gt_body",u"e_gte_body",b"e_gte_body",u"e_list_body",b"e_list_body",u"e_lt_body",b"e_lt_body",u"e_lte_body",b"e_lte_body",u"e_map_body",b"e_map_body",u"e_matches_body",b"e_matches_body",u"e_method_body",b"e_method_body",u"e_minus_body",b"e_minus_body",u"e_minus_minus_body",b"e_minus_minus_body",u"e_mod_body",b"e_mod_body",u"e_mult_body",b"e_mult_body",u"e_neg_body",b"e_neg_body",u"e_neq_body",b"e_neq_body",u"e_not_body",b"e_not_body",u"e_or_body",b"e_or_body",u"e_percent_percent_body",b"e_percent_percent_body",u"e_plus_body",b"e_plus_body",u"e_plus_plus_body",b"e_plus_plus_body",u"e_set_body",b"e_set_body",u"e_tuple_body",b"e_tuple_body",u"e_var_body",b"e_var_body",u"expr_instance",b"expr_instance",u"g_bool",b"g_bool",u"g_byte_array",b"g_byte_array",u"g_int",b"g_int",u"g_string",b"g_string",u"g_uri",b"g_uri"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"e_and_body",b"e_and_body",u"e_div_body",b"e_div_body",u"e_eq_body",b"e_eq_body",u"e_gt_body",b"e_gt_body",u"e_gte_body",b"e_gte_body",u"e_list_body",b"e_list_body",u"e_lt_body",b"e_lt_body",u"e_lte_body",b"e_lte_body",u"e_map_body",b"e_map_body",u"e_matches_body",b"e_matches_body",u"e_method_body",b"e_method_body",u"e_minus_body",b"e_minus_body",u"e_minus_minus_body",b"e_minus_minus_body",u"e_mod_body",b"e_mod_body",u"e_mult_body",b"e_mult_body",u"e_neg_body",b"e_neg_body",u"e_neq_body",b"e_neq_body",u"e_not_body",b"e_not_body",u"e_or_body",b"e_or_body",u"e_percent_percent_body",b"e_percent_percent_body",u"e_plus_body",b"e_plus_body",u"e_plus_plus_body",b"e_plus_plus_body",u"e_set_body",b"e_set_body",u"e_tuple_body",b"e_tuple_body",u"e_var_body",b"e_var_body",u"expr_instance",b"expr_instance",u"g_bool",b"g_bool",u"g_byte_array",b"g_byte_array",u"g_int",b"g_int",u"g_string",b"g_string",u"g_uri",b"g_uri"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"expr_instance",b"expr_instance"]) -> typing_extensions___Literal["g_bool","g_int","g_string","g_uri","g_byte_array","e_not_body","e_neg_body","e_mult_body","e_div_body","e_plus_body","e_minus_body","e_lt_body","e_lte_body","e_gt_body","e_gte_body","e_eq_body","e_neq_body","e_and_body","e_or_body","e_var_body","e_list_body","e_tuple_body","e_set_body","e_map_body","e_method_body","e_matches_body","e_percent_percent_body","e_plus_plus_body","e_minus_minus_body","e_mod_body"]: ...

class EList(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    locallyFree = ... # type: builtin___bytes
    connective_used = ... # type: builtin___bool

    @property
    def ps(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Par]: ...

    @property
    def remainder(self) -> Var: ...

    def __init__(self,
        *,
        ps : typing___Optional[typing___Iterable[Par]] = None,
        locallyFree : typing___Optional[builtin___bytes] = None,
        connective_used : typing___Optional[builtin___bool] = None,
        remainder : typing___Optional[Var] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EList: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EList: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"remainder",b"remainder"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"connective_used",b"connective_used",u"locallyFree",b"locallyFree",u"ps",b"ps",u"remainder",b"remainder"]) -> None: ...

class ETuple(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    locallyFree = ... # type: builtin___bytes
    connective_used = ... # type: builtin___bool

    @property
    def ps(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Par]: ...

    def __init__(self,
        *,
        ps : typing___Optional[typing___Iterable[Par]] = None,
        locallyFree : typing___Optional[builtin___bytes] = None,
        connective_used : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ETuple: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ETuple: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"connective_used",b"connective_used",u"locallyFree",b"locallyFree",u"ps",b"ps"]) -> None: ...

class ESet(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    locallyFree = ... # type: builtin___bytes
    connective_used = ... # type: builtin___bool

    @property
    def ps(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Par]: ...

    @property
    def remainder(self) -> Var: ...

    def __init__(self,
        *,
        ps : typing___Optional[typing___Iterable[Par]] = None,
        locallyFree : typing___Optional[builtin___bytes] = None,
        connective_used : typing___Optional[builtin___bool] = None,
        remainder : typing___Optional[Var] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ESet: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ESet: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"remainder",b"remainder"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"connective_used",b"connective_used",u"locallyFree",b"locallyFree",u"ps",b"ps",u"remainder",b"remainder"]) -> None: ...

class EMap(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    locallyFree = ... # type: builtin___bytes
    connective_used = ... # type: builtin___bool

    @property
    def kvs(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[KeyValuePair]: ...

    @property
    def remainder(self) -> Var: ...

    def __init__(self,
        *,
        kvs : typing___Optional[typing___Iterable[KeyValuePair]] = None,
        locallyFree : typing___Optional[builtin___bytes] = None,
        connective_used : typing___Optional[builtin___bool] = None,
        remainder : typing___Optional[Var] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EMap: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EMap: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"remainder",b"remainder"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"connective_used",b"connective_used",u"kvs",b"kvs",u"locallyFree",b"locallyFree",u"remainder",b"remainder"]) -> None: ...

class EMethod(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    methodName = ... # type: typing___Text
    locallyFree = ... # type: builtin___bytes
    connective_used = ... # type: builtin___bool

    @property
    def target(self) -> Par: ...

    @property
    def arguments(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Par]: ...

    def __init__(self,
        *,
        methodName : typing___Optional[typing___Text] = None,
        target : typing___Optional[Par] = None,
        arguments : typing___Optional[typing___Iterable[Par]] = None,
        locallyFree : typing___Optional[builtin___bytes] = None,
        connective_used : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EMethod: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EMethod: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"target",b"target"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"arguments",b"arguments",u"connective_used",b"connective_used",u"locallyFree",b"locallyFree",u"methodName",b"methodName",u"target",b"target"]) -> None: ...

class KeyValuePair(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def key(self) -> Par: ...

    @property
    def value(self) -> Par: ...

    def __init__(self,
        *,
        key : typing___Optional[Par] = None,
        value : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> KeyValuePair: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> KeyValuePair: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"key",b"key",u"value",b"value"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"key",b"key",u"value",b"value"]) -> None: ...

class EVar(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def v(self) -> Var: ...

    def __init__(self,
        *,
        v : typing___Optional[Var] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EVar: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EVar: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"v",b"v"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"v",b"v"]) -> None: ...

class ENot(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p(self) -> Par: ...

    def __init__(self,
        *,
        p : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ENot: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ENot: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p",b"p"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p",b"p"]) -> None: ...

class ENeg(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p(self) -> Par: ...

    def __init__(self,
        *,
        p : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ENeg: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ENeg: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p",b"p"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p",b"p"]) -> None: ...

class EMult(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EMult: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EMult: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class EDiv(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EDiv: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EDiv: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class EMod(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EMod: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EMod: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class EPlus(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EPlus: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EPlus: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class EMinus(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EMinus: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EMinus: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class ELt(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ELt: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ELt: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class ELte(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ELte: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ELte: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class EGt(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EGt: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EGt: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class EGte(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EGte: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EGte: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class EEq(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EEq: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EEq: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class ENeq(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ENeq: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ENeq: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class EAnd(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EAnd: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EAnd: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class EOr(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EOr: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EOr: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class EMatches(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def target(self) -> Par: ...

    @property
    def pattern(self) -> Par: ...

    def __init__(self,
        *,
        target : typing___Optional[Par] = None,
        pattern : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EMatches: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EMatches: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"pattern",b"pattern",u"target",b"target"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"pattern",b"pattern",u"target",b"target"]) -> None: ...

class EPercentPercent(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EPercentPercent: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EPercentPercent: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class EPlusPlus(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EPlusPlus: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EPlusPlus: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class EMinusMinus(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def p1(self) -> Par: ...

    @property
    def p2(self) -> Par: ...

    def __init__(self,
        *,
        p1 : typing___Optional[Par] = None,
        p2 : typing___Optional[Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EMinusMinus: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EMinusMinus: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"p1",b"p1",u"p2",b"p2"]) -> None: ...

class Connective(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    conn_bool = ... # type: builtin___bool
    conn_int = ... # type: builtin___bool
    conn_string = ... # type: builtin___bool
    conn_uri = ... # type: builtin___bool
    conn_byte_array = ... # type: builtin___bool

    @property
    def conn_and_body(self) -> ConnectiveBody: ...

    @property
    def conn_or_body(self) -> ConnectiveBody: ...

    @property
    def conn_not_body(self) -> Par: ...

    @property
    def var_ref_body(self) -> VarRef: ...

    def __init__(self,
        *,
        conn_and_body : typing___Optional[ConnectiveBody] = None,
        conn_or_body : typing___Optional[ConnectiveBody] = None,
        conn_not_body : typing___Optional[Par] = None,
        var_ref_body : typing___Optional[VarRef] = None,
        conn_bool : typing___Optional[builtin___bool] = None,
        conn_int : typing___Optional[builtin___bool] = None,
        conn_string : typing___Optional[builtin___bool] = None,
        conn_uri : typing___Optional[builtin___bool] = None,
        conn_byte_array : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Connective: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Connective: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"conn_and_body",b"conn_and_body",u"conn_bool",b"conn_bool",u"conn_byte_array",b"conn_byte_array",u"conn_int",b"conn_int",u"conn_not_body",b"conn_not_body",u"conn_or_body",b"conn_or_body",u"conn_string",b"conn_string",u"conn_uri",b"conn_uri",u"connective_instance",b"connective_instance",u"var_ref_body",b"var_ref_body"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"conn_and_body",b"conn_and_body",u"conn_bool",b"conn_bool",u"conn_byte_array",b"conn_byte_array",u"conn_int",b"conn_int",u"conn_not_body",b"conn_not_body",u"conn_or_body",b"conn_or_body",u"conn_string",b"conn_string",u"conn_uri",b"conn_uri",u"connective_instance",b"connective_instance",u"var_ref_body",b"var_ref_body"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"connective_instance",b"connective_instance"]) -> typing_extensions___Literal["conn_and_body","conn_or_body","conn_not_body","var_ref_body","conn_bool","conn_int","conn_string","conn_uri","conn_byte_array"]: ...

class VarRef(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    index = ... # type: builtin___int
    depth = ... # type: builtin___int

    def __init__(self,
        *,
        index : typing___Optional[builtin___int] = None,
        depth : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> VarRef: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> VarRef: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"depth",b"depth",u"index",b"index"]) -> None: ...

class ConnectiveBody(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def ps(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Par]: ...

    def __init__(self,
        *,
        ps : typing___Optional[typing___Iterable[Par]] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ConnectiveBody: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ConnectiveBody: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"ps",b"ps"]) -> None: ...

class DeployId(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    sig = ... # type: builtin___bytes

    def __init__(self,
        *,
        sig : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> DeployId: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> DeployId: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"sig",b"sig"]) -> None: ...

class DeployerId(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    publicKey = ... # type: builtin___bytes

    def __init__(self,
        *,
        publicKey : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> DeployerId: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> DeployerId: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"publicKey",b"publicKey"]) -> None: ...

class GUnforgeable(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def g_private_body(self) -> GPrivate: ...

    @property
    def g_deploy_id_body(self) -> GDeployId: ...

    @property
    def g_deployer_id_body(self) -> GDeployerId: ...

    @property
    def g_sys_auth_token_body(self) -> GSysAuthToken: ...

    def __init__(self,
        *,
        g_private_body : typing___Optional[GPrivate] = None,
        g_deploy_id_body : typing___Optional[GDeployId] = None,
        g_deployer_id_body : typing___Optional[GDeployerId] = None,
        g_sys_auth_token_body : typing___Optional[GSysAuthToken] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> GUnforgeable: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> GUnforgeable: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"g_deploy_id_body",b"g_deploy_id_body",u"g_deployer_id_body",b"g_deployer_id_body",u"g_private_body",b"g_private_body",u"g_sys_auth_token_body",b"g_sys_auth_token_body",u"unf_instance",b"unf_instance"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"g_deploy_id_body",b"g_deploy_id_body",u"g_deployer_id_body",b"g_deployer_id_body",u"g_private_body",b"g_private_body",u"g_sys_auth_token_body",b"g_sys_auth_token_body",u"unf_instance",b"unf_instance"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"unf_instance",b"unf_instance"]) -> typing_extensions___Literal["g_private_body","g_deploy_id_body","g_deployer_id_body","g_sys_auth_token_body"]: ...

class GPrivate(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    id = ... # type: builtin___bytes

    def __init__(self,
        *,
        id : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> GPrivate: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> GPrivate: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"id",b"id"]) -> None: ...

class GDeployId(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    sig = ... # type: builtin___bytes

    def __init__(self,
        *,
        sig : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> GDeployId: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> GDeployId: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"sig",b"sig"]) -> None: ...

class GDeployerId(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    publicKey = ... # type: builtin___bytes

    def __init__(self,
        *,
        publicKey : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> GDeployerId: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> GDeployerId: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"publicKey",b"publicKey"]) -> None: ...

class GSysAuthToken(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    def __init__(self,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> GSysAuthToken: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> GSysAuthToken: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from .RhoTypes_pb2 import (
    BindPattern as RhoTypes_pb2___BindPattern,
    Par as RhoTypes_pb2___Par,
)

from google.protobuf.descriptor import (
    Descriptor as google___protobuf___descriptor___Descriptor,
)

from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer as google___protobuf___internal___containers___RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer as google___protobuf___internal___containers___RepeatedScalarFieldContainer,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Iterable as typing___Iterable,
    Optional as typing___Optional,
    Text as typing___Text,
    Union as typing___Union,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


builtin___bool = bool
builtin___bytes = bytes
builtin___float = float
builtin___int = int
if sys.version_info < (3,):
    builtin___buffer = buffer
    builtin___unicode = unicode


class FindDeployQuery(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    deployId = ... # type: builtin___bytes

    def __init__(self,
        *,
        deployId : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> FindDeployQuery: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> FindDeployQuery: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"deployId",b"deployId"]) -> None: ...

class BlockQuery(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    hash = ... # type: typing___Text

    def __init__(self,
        *,
        hash : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BlockQuery: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BlockQuery: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"hash",b"hash"]) -> None: ...

class BlocksQuery(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    depth = ... # type: builtin___int

    def __init__(self,
        *,
        depth : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BlocksQuery: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BlocksQuery: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"depth",b"depth"]) -> None: ...

class DataAtNameQuery(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    depth = ... # type: builtin___int

    @property
    def name(self) -> RhoTypes_pb2___Par: ...

    def __init__(self,
        *,
        depth : typing___Optional[builtin___int] = None,
        name : typing___Optional[RhoTypes_pb2___Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> DataAtNameQuery: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> DataAtNameQuery: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"name",b"name"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"depth",b"depth",u"name",b"name"]) -> None: ...

class ContinuationAtNameQuery(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    depth = ... # type: builtin___int

    @property
    def names(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[RhoTypes_pb2___Par]: ...

    def __init__(self,
        *,
        depth : typing___Optional[builtin___int] = None,
        names : typing___Optional[typing___Iterable[RhoTypes_pb2___Par]] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ContinuationAtNameQuery: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ContinuationAtNameQuery: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"depth",b"depth",u"names",b"names"]) -> None: ...

class VisualizeDagQuery(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    depth = ... # type: builtin___int
    showJustificationLines = ... # type: builtin___bool

    def __init__(self,
        *,
        depth : typing___Optional[builtin___int] = None,
        showJustificationLines : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> VisualizeDagQuery: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> VisualizeDagQuery: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"depth",b"depth",u"showJustificationLines",b"showJustificationLines"]) -> None: ...

class MachineVerifyQuery(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    def __init__(self,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> MachineVerifyQuery: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> MachineVerifyQuery: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class PrivateNamePreviewQuery(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    user = ... # type: builtin___bytes
    timestamp = ... # type: builtin___int
    nameQty = ... # type: builtin___int

    def __init__(self,
        *,
        user : typing___Optional[builtin___bytes] = None,
        timestamp : typing___Optional[builtin___int] = None,
        nameQty : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> PrivateNamePreviewQuery: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> PrivateNamePreviewQuery: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"nameQty",b"nameQty",u"timestamp",b"timestamp",u"user",b"user"]) -> None: ...

class LastFinalizedBlockQuery(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    def __init__(self,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> LastFinalizedBlockQuery: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> LastFinalizedBlockQuery: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class IsFinalizedQuery(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    hash = ... # type: typing___Text

    def __init__(self,
        *,
        hash : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> IsFinalizedQuery: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> IsFinalizedQuery: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"hash",b"hash"]) -> None: ...

class BondStatusQuery(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    publicKey = ... # type: builtin___bytes

    def __init__(self,
        *,
        publicKey : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BondStatusQuery: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BondStatusQuery: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"publicKey",b"publicKey"]) -> None: ...

class ExploratoryDeployQuery(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    term = ... # type: typing___Text

    def __init__(self,
        *,
        term : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ExploratoryDeployQuery: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ExploratoryDeployQuery: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"term",b"term"]) -> None: ...

class BondInfo(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    validator = ... # type: typing___Text
    stake = ... # type: builtin___int

    def __init__(self,
        *,
        validator : typing___Optional[typing___Text] = None,
        stake : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BondInfo: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BondInfo: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"stake",b"stake",u"validator",b"validator"]) -> None: ...

class DeployInfo(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    deployer = ... # type: typing___Text
    term = ... # type: typing___Text
    timestamp = ... # type: builtin___int
    sig = ... # type: typing___Text
    sigAlgorithm = ... # type: typing___Text
    phloPrice = ... # type: builtin___int
    phloLimit = ... # type: builtin___int
    validAfterBlockNumber = ... # type: builtin___int
    cost = ... # type: builtin___int
    errored = ... # type: builtin___bool
    systemDeployError = ... # type: typing___Text

    def __init__(self,
        *,
        deployer : typing___Optional[typing___Text] = None,
        term : typing___Optional[typing___Text] = None,
        timestamp : typing___Optional[builtin___int] = None,
        sig : typing___Optional[typing___Text] = None,
        sigAlgorithm : typing___Optional[typing___Text] = None,
        phloPrice : typing___Optional[builtin___int] = None,
        phloLimit : typing___Optional[builtin___int] = None,
        validAfterBlockNumber : typing___Optional[builtin___int] = None,
        cost : typing___Optional[builtin___int] = None,
        errored : typing___Optional[builtin___bool] = None,
        systemDeployError : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> DeployInfo: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> DeployInfo: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"cost",b"cost",u"deployer",b"deployer",u"errored",b"errored",u"phloLimit",b"phloLimit",u"phloPrice",b"phloPrice",u"sig",b"sig",u"sigAlgorithm",b"sigAlgorithm",u"systemDeployError",b"systemDeployError",u"term",b"term",u"timestamp",b"timestamp",u"validAfterBlockNumber",b"validAfterBlockNumber"]) -> None: ...

class LightBlockInfo(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    blockHash = ... # type: typing___Text
    sender = ... # type: typing___Text
    seqNum = ... # type: builtin___int
    sig = ... # type: typing___Text
    sigAlgorithm = ... # type: typing___Text
    shardId = ... # type: typing___Text
    extraBytes = ... # type: builtin___bytes
    version = ... # type: builtin___int
    timestamp = ... # type: builtin___int
    headerExtraBytes = ... # type: builtin___bytes
    parentsHashList = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]
    blockNumber = ... # type: builtin___int
    preStateHash = ... # type: typing___Text
    postStateHash = ... # type: typing___Text
    bodyExtraBytes = ... # type: builtin___bytes
    blockSize = ... # type: typing___Text
    deployCount = ... # type: builtin___int
    faultTolerance = ... # type: builtin___float

    @property
    def bonds(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[BondInfo]: ...

    def __init__(self,
        *,
        blockHash : typing___Optional[typing___Text] = None,
        sender : typing___Optional[typing___Text] = None,
        seqNum : typing___Optional[builtin___int] = None,
        sig : typing___Optional[typing___Text] = None,
        sigAlgorithm : typing___Optional[typing___Text] = None,
        shardId : typing___Optional[typing___Text] = None,
        extraBytes : typing___Optional[builtin___bytes] = None,
        version : typing___Optional[builtin___int] = None,
        timestamp : typing___Optional[builtin___int] = None,
        headerExtraBytes : typing___Optional[builtin___bytes] = None,
        parentsHashList : typing___Optional[typing___Iterable[typing___Text]] = None,
        blockNumber : typing___Optional[builtin___int] = None,
        preStateHash : typing___Optional[typing___Text] = None,
        postStateHash : typing___Optional[typing___Text] = None,
        bodyExtraBytes : typing___Optional[builtin___bytes] = None,
        bonds : typing___Optional[typing___Iterable[BondInfo]] = None,
        blockSize : typing___Optional[typing___Text] = None,
        deployCount : typing___Optional[builtin___int] = None,
        faultTolerance : typing___Optional[builtin___float] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> LightBlockInfo: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> LightBlockInfo: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"blockHash",b"blockHash",u"blockNumber",b"blockNumber",u"blockSize",b"blockSize",u"bodyExtraBytes",b"bodyExtraBytes",u"bonds",b"bonds",u"deployCount",b"deployCount",u"extraBytes",b"extraBytes",u"faultTolerance",b"faultTolerance",u"headerExtraBytes",b"headerExtraBytes",u"parentsHashList",b"parentsHashList",u"postStateHash",b"postStateHash",u"preStateHash",b"preStateHash",u"sender",b"sender",u"seqNum",b"seqNum",u"shardId",b"shardId",u"sig",b"sig",u"sigAlgorithm",b"sigAlgorithm",u"timestamp",b"timestamp",u"version",b"version"]) -> None: ...

class BlockInfo(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def blockInfo(self) -> LightBlockInfo: ...

    @property
    def deploys(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[DeployInfo]: ...

    def __init__(self,
        *,
        blockInfo : typing___Optional[LightBlockInfo] = None,
        deploys : typing___Optional[typing___Iterable[DeployInfo]] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BlockInfo: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BlockInfo: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"blockInfo",b"blockInfo"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"blockInfo",b"blockInfo",u"deploys",b"deploys"]) -> None: ...

class DataWithBlockInfo(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def postBlockData(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[RhoTypes_pb2___Par]: ...

    @property
    def block(self) -> LightBlockInfo: ...

    def __init__(self,
        *,
        postBlockData : typing___Optional[typing___Iterable[RhoTypes_pb2___Par]] = None,
        block : typing___Optional[LightBlockInfo] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> DataWithBlockInfo: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> DataWithBlockInfo: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"block",b"block"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"block",b"block",u"postBlockData",b"postBlockData"]) -> None: ...

class ContinuationsWithBlockInfo(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def postBlockContinuations(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[WaitingContinuationInfo]: ...

    @property
    def block(self) -> LightBlockInfo: ...

    def __init__(self,
        *,
        postBlockContinuations : typing___Optional[typing___Iterable[WaitingContinuationInfo]] = None,
        block : typing___Optional[LightBlockInfo] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ContinuationsWithBlockInfo: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ContinuationsWithBlockInfo: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"block",b"block"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"block",b"block",u"postBlockContinuations",b"postBlockContinuations"]) -> None: ...

class WaitingContinuationInfo(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def postBlockPatterns(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[RhoTypes_pb2___BindPattern]: ...

    @property
    def postBlockContinuation(self) -> RhoTypes_pb2___Par: ...

    def __init__(self,
        *,
        postBlockPatterns : typing___Optional[typing___Iterable[RhoTypes_pb2___BindPattern]] = None,
        postBlockContinuation : typing___Optional[RhoTypes_pb2___Par] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> WaitingContinuationInfo: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> WaitingContinuationInfo: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"postBlockContinuation",b"postBlockContinuation"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"postBlockContinuation",b"postBlockContinuation",u"postBlockPatterns",b"postBlockPatterns"]) -> None: ...
